#ifndef _IPL_REG_H_
#define _IPL_REG_H_

#include "io.h"

#define SRAM_START		(0xe6300000)
#define SRAM_SIZE		(0x00040000)	/* 256kB */

#define SRAM_IPL_START		(SRAM_START)
#define SRAM_IPL_SIZE		(0x00004000)	/* 16kB */

#define ICRAM0_START		(0xe63a0000)
#define ICRAM0_SIZE		(0x00012000)	/* 72kB */

#define ICRAM0_IPL_START	(ICRAM0_START)
#define ICRAM0_IPL_SIZE		(0x00004000)	/* 16kB */

#define ICRAM1_START		(0xe63c0000)
#define ICRAM1_SIZE		(0x00001000)	/* 4kB */

#define SRAM_HERMIT_START	(SRAM_START)
#define SRAM_HERMIT_SIZE	(0x00040000)	/* 256kB */

#define QSPI_HERMIT_START	(0x00010000)
#define QSPI_HERMIT_SIZE	(0x00040000)	/* 256kB */

#define STACK_FIQ_START		(0xe63b2000)
#define STACK_IRQ_START		(0xe63b1f00)
#define STACK_SVC_START		(0xe63b1e00)
#define STACK_ABT_START		(0xe63b1d00)
#define STACK_UND_START		(0xe63b1c00)
#define STACK_SYS_START		(0xe63b1b00)

/* RCLK Watchdog timer(RWDT) */
#define RWTCNT			(0xe6020000)
#define RWTCSRA			(0xe6020004)
#define RWTCSRB			(0xe6020008)

/* Secure Watchdog Timer */
#define SWTCNT			(0xe6030000)
#define SWTCSRA			(0xe6030004)
#define SWTCSRB			(0xe6030008)

/* Pin Function Controller(PFC) */
#define PFC_PMMR		(0xe6060000)

#define PFC_GPSR0		(0xe6060004)
#define PFC_GPSR1		(0xe6060008)
#define PFC_GPSR2		(0xe606000c)
#define PFC_GPSR3		(0xe6060010)
#define PFC_GPSR4		(0xe6060014)
#define PFC_GPSR5		(0xe6060018)
#define PFC_GPSR6		(0xe606001c)
#define PFC_GPSR7		(0xe6060074)

#define PFC_IPSR0		(0xe6060020)
#define PFC_IPSR1		(0xe6060024)
#define PFC_IPSR2		(0xe6060028)
#define PFC_IPSR3		(0xe606002c)
#define PFC_IPSR4		(0xe6060030)
#define PFC_IPSR5		(0xe6060034)
#define PFC_IPSR6		(0xe6060038)
#define PFC_IPSR7		(0xe606003c)
#define PFC_IPSR8		(0xe6060040)
#define PFC_IPSR9		(0xe6060044)
#define PFC_IPSR10		(0xe6060048)
#define PFC_IPSR11		(0xe606004c)
#define PFC_IPSR12		(0xe6060050)
#define PFC_IPSR13		(0xe6060054)
#define PFC_IPSR14		(0xe6060058)
#define PFC_IPSR15		(0xe606005c)
#define PFC_IPSR16		(0xe6060160)

#define PFC_MOD_SEL		(0xe6060090)
#define PFC_MOD_SEL2		(0xe6060094)
#define PFC_MOD_SEL3		(0xe6060098)
#define PFC_MOD_SEL4		(0xe606009c)

#define PFC_PUPR0		(0xe6060100)
#define PFC_PUPR1		(0xe6060104)
#define PFC_PUPR2		(0xe6060108)
#define PFC_PUPR3		(0xe606010c)
#define PFC_PUPR4		(0xe6060110)
#define PFC_PUPR5		(0xe6060114)
#define PFC_PUPR6		(0xe6060118)
#define PFC_PUPR7		(0xe606011c)

/* GPIO */
#define GPIO0_BASE_ADDR		0xe6050000
#define GPIO1_BASE_ADDR		0xe6051000
#define GPIO2_BASE_ADDR		0xe6052000
#define GPIO3_BASE_ADDR		0xe6053000
#define GPIO4_BASE_ADDR		0xe6054000
#define GPIO5_BASE_ADDR		0xe6055000
#define GPIO6_BASE_ADDR		0xe6055400
#define GPIO7_BASE_ADDR		0xe6055800

#define GPIO_IOINTSEL		(0x00)
#define GPIO_INOUTSEL		(0x04)
#define GPIO_OUTDT		(0x08)
#define GPIO_INDT		(0x0c)
#define GPIO_INTDT		(0x10)
#define GPIO_INTCLR		(0x14)
#define GPIO_INTMSK		(0x18)
#define GPIO_MSKCLR		(0x1c)
#define GPIO_POSNEG		(0x20)
#define GPIO_EDGLEVEL		(0x24)
#define GPIO_FILONOFF		(0x28)
#define GPIO_INTMSKS		(0x38)
#define GPIO_MSKCLRS		(0x3c)
#define GPIO_OUTDTSEL		(0x40)
#define GPIO_OUTDTH		(0x44)
#define GPIO_OUTDTL		(0x48)
#define GPIO_BOTHEDGE		(0x4c)

/* SCIF */
#define SCIF0_BASE_ADDR		(0xe6e60000)
#define SCIF1_BASE_ADDR		(0xe6e68000)

#define SCIF_SCSMR		(0x00)
#define SCIF_SCBRR		(0x04)
#define SCIF_SCSCR		(0x08)
#define SCIF_SCFTDR		(0x0c)
#define SCIF_SCFSR		(0x10)
#define		SCIF_SCFSR_ER		(1<<7)
#define		SCIF_SCFSR_TEND		(1<<6)
#define		SCIF_SCFSR_TDFE		(1<<5)
#define		SCIF_SCFSR_BRK		(1<<4)
#define		SCIF_SCFSR_FER		(1<<3)
#define		SCIF_SCFSR_PER		(1<<2)
#define		SCIF_SCFSR_RDF		(1<<1)
#define		SCIF_SCFSR_DR		(1<<0)
#define SCIF_SCFRDR		(0x14)
#define SCIF_SCFCR		(0x18)
#define SCIF_SCFDR		(0x1c)
#define SCIF_SCSPTR		(0x20)
#define SCIF_SCLSR		(0x24)
#define		SCIF_SCLSR_ORER		(1<<0)
#define SCIF_DL			(0x30)
#define SCIF_CKS		(0x34)

/* SCIFB */
#define SCIFB0_BASE_ADDR	(0xe6c20000)
#define SCIFB1_BASE_ADDR	(0xe6c30000)
#define SCIFB2_BASE_ADDR	(0xe6ce0000)

#define SCIFB_SCBSMR		(0x00)
#define SCIFB_SCBBRR		(0x04)
#define SCIFB_SCBSCR		(0x08)
#define SCIFB_SCBFTDR		(0x40)
#define SCIFB_SCBFSR		(0x14)
#define		SCIFB_SCBFSR_ER		(1<<7)
#define		SCIFB_SCBFSR_TEND	(1<<6)
#define		SCIFB_SCBFSR_TDFE	(1<<5)
#define		SCIFB_SCBFSR_BRK	(1<<4)
#define		SCIFB_SCBFSR_FER	(1<<3)
#define		SCIFB_SCBFSR_PER	(1<<2)
#define		SCIFB_SCBFSR_RDF	(1<<1)
#define		SCIFB_SCBFSR_DR		(1<<0)
#define SCIFB_SCBFRDR		(0x60)
#define SCIFB_SCBFCR		(0x18)
#define SCIFB_SCBTFDR		(0x38)
#define SCIFB_SCBRFDR		(0x3c)


#define SCIF_DL_VALUE(bps, clk)	(clk / (bps * 16))

/* CPG (see Table 7A-3) */
#define CPG_MSTPSR1		(0xe6150038)
#define CPG_SMSTPCR1	(0xe6150134)
#define CPG_RMSTPCR1	(0xe6150114)
#define 	CPG_MSTP1_TMU0		(1<<25)

#define CPG_MSTPSR2		(0xe6150040)
#define CPG_SMSTPCR2		(0xe6150138)
#define 	CPG_MSTP2_SCIFB1	(1<<7)
#define 	CPG_MSTP2_SCIFB0	(1<<6)

#define CPG_MSTPSR3		(0xe6150048)
#define CPG_SMSTPCR3	(0xe615013c)
#define CPG_RMSTPCR3	(0xe615011c)
#define 	CPG_MSTP3_MMC0		(1<<15)
#define 	CPG_MSTP3_SDHI0		(1<<14)
#define 	CPG_MSTP3_SDHI1		(1<<12)
#define 	CPG_MSTP3_SDHI2		(1<<11)

#define CPG_MSTPSR7		(0xe61501c4)
#define CPG_SMSTPCR7		(0xe615014c)
#define 	CPG_MSTP7_SCIF1	(1<<20)
#define 	CPG_MSTP7_SCIF0	(1<<21)

#define CPG_MSTPSR9		(0xe61509a4)
#define	CPG_SMSTPCR9		(0xe6150994)
#define 	CPG_MSTP9_QSPI	(1<<17)

/* Quad Serial Peripheral Interface(QSPI) */
#define	QSPI_SPCR		(0xe6b10000)
#define		QSPI_SPCR_SPRIE		(1<<7)
#define		QSPI_SPCR_SPE		(1<<6)
#define		QSPI_SPCR_SPTIE		(1<<5)
#define		QSPI_SPCR_SPEIE		(1<<4)
#define		QSPI_SPCR_MSTR		(1<<3)
#define		QSPI_SPCR_WSWAP		(1<<1)
#define		QSPI_SPCR_BSWAP		(1<<0)
#define	QSPI_SSLP		(0xe6b10001)
#define		QSPI_SSLP_SSLP		(1<<0)
#define	QSPI_SPPCR		(0xe6b10002)
#define		QSPI_SPPCR_MOIFE	(1<<5)
#define		QSPI_SPPCR_MOIFV	(1<<4)
#define		QSPI_SPPCR_IO3FV	(1<<2)
#define		QSPI_SPPCR_IO2FV	(1<<1)
#define		QSPI_SPPCR_SPLP		(1<<0)
#define	QSPI_SPSR		(0xe6b10003)
#define		QSPI_SPSR_SPRFF		(1<<7)
#define		QSPI_SPSR_TEND		(1<<6)
#define		QSPI_SPSR_SPTEF		(1<<5)
#define	QSPI_SPDR		(0xe6b10004)
#define	QSPI_SPSCR		(0xe6b10008)
#define		QSPI_SPSCR_SPSC_MASK	(0x03)
#define	QSPI_SPSSR		(0xe6b10009)
#define	QSPI_SPBR		(0xe6b1000a)
#define	QSPI_SPDCR		(0xe6b1000b)
#define		QSPI_SPDCR_TXDMY	(1<<7)
#define	QSPI_SPCKD		(0xe6b1000c)
#define	QSPI_SSLND		(0xe6b1000d)
#define	QSPI_SPND		(0xe6b1000e)
#define	QSPI_SPCMD0		(0xe6b10010)
#define	QSPI_SPCMD1		(0xe6b10012)
#define	QSPI_SPCMD2		(0xe6b10014)
#define	QSPI_SPCMD3		(0xe6b10016)
#define		QSPI_SPCMD_SCKDEN	(1<<15)
#define		QSPI_SPCMD_SLNDEN	(1<<14)
#define		QSPI_SPCMD_SPNDEN	(1<<13)
#define		QSPI_SPCMD_LSBF		(1<<12)
#define		QSPI_SPCMD_SPB_MASK	(0x0f00)
#define		QSPI_SPCMD_SSLKP	(1<<7)
#define		QSPI_SPCMD_SPIMOD_MASK	(0x0060)
#define		QSPI_SPCMD_SPRW		(1<<4)
#define		QSPI_SPCMD_BRDV_MASK	(0x000c)
#define		QSPI_SPCMD_BRDV_DIV8	(0x000c)
#define		QSPI_SPCMD_BRDV_DIV4	(0x0008)
#define		QSPI_SPCMD_BRDV_DIV2	(0x0004)
#define		QSPI_SPCMD_BRDV_DIV0	(0x0000)

#define		QSPI_SPCMD_CPOL		(1<<1)
#define		QSPI_SPCMD_CPHA		(1<<0)
#define	QSPI_SPBFCR		(0xe6b10018)
#define		QSPI_SPBFCR_TXRST	(1<<7)
#define		QSPI_SPBFCR_RXRST	(1<<6)
#define		QSPI_SPBFCR_TXTRG_MASK	(0x30)
#define		QSPI_SPBFCR_TXTRG_31B	(0x00)
#define		QSPI_SPBFCR_TXTRG_0B	(0x30)
#define		QSPI_SPBFCR_RXTRG_MASK	(0x07)
#define		QSPI_SPBFCR_RXTRG_1B	(0x00)
#define		QSPI_SPBFCR_RXTRG_2B	(0x01)
#define		QSPI_SPBFCR_RXTRG_4B	(0x02)
#define		QSPI_SPBFCR_RXTRG_8B	(0x04)
#define		QSPI_SPBFCR_RXTRG_32B	(0x07)
#define	QSPI_SPBDCR		(0xe6b1001a)
#define	QSPI_SPBMUL0		(0xe6b1001c)
#define	QSPI_SPBMUL1		(0xe6b10020)
#define	QSPI_SPBMUL2		(0xe6b10024)
#define	QSPI_SPBMUL3		(0xe6b10028)

#define QSPI_BUFFER_SIZE	(32u)

/* TMU */
#define TMU_TSTR0		0xe61e0004

#define TMU0_BASE_ADDR		0xe61e0008
#define TMU1_BASE_ADDR		0xe61e0014
#define TMU2_BASE_ADDR		0xe61e0020
#define TMU(PORT, REG)		(TMU##PORT##_BASE_ADDR + TMU_##REG)
#define TMU_TCOR		0x00
#define TMU_TCNT		0x04
#define TMU_TCR			0x08

#ifndef __ASSEMBLY__

#include "gpio.h"

/* Boot device select */
#define sw_sdboot_en()	(gpio_get_value(PIN(2, 3)))	/* SD_BOOT_EN_N*/
#define sw_sdhi0_en()	(!!gpio_get_value(PIN(5, 31)))	/* IPL_EN */

void sdhi_voltage_init(void);
void alarm_led_off(void);
#endif

#endif
