/*
 * boot.S: assembly language bootstrap
 *
 * Copyright (c) 2014 Atmark Techno, Inc.
 */

#include "board.h"
#include "arm.h"

	.section .pre.boot
	.global _start
_start:
	/****************************************************************
	 * Register
	 ****************************************************************/
	ldr	r0, =0
	ldr	r1, =0
	ldr	r2, =0
	ldr	r3, =0
	ldr	r4, =0
	ldr	r5, =0
	ldr	r6, =0
	ldr	r7, =0
	ldr	r8, =0
	ldr	r9, =0
	ldr	r10, =0
	ldr	r11, =0
	ldr	r12, =0
	ldr	r13, =0
	ldr	r14, =0

	/****************************************************************
	 * Stack
	 ****************************************************************/
	/* IRQ */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_IRQ_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_IRQ_START
	mov	sp, r0

	/* FIQ */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_FIQ_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_FIQ_START
	mov	sp, r0

	/* ABT */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_ABT_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_ABT_START
	mov	sp, r0

	/* SVC */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_SVC_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_SVC_START
	mov	sp, r0

	/* UND */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_UND_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_UND_START
	mov	sp, r0

	/* SYS */
	mrs	r1, cpsr
	bic	r1, r1, #PSR_MODE_MASK
	orr	r1, r1, #PSR_SYS_MODE
	msr	cpsr_c, r1

	ldr	r0, =STACK_SYS_START
	mov	sp, r0

	/****************************************************************
	 * WDT init
	 ****************************************************************/
	/* disable RCLK Watchdog timer */
	ldr	r0, =RWTCSRA
	ldr	r1, =0xa5a5a500
	str	r1, [r0]

	/* disable Secure Watchdog Timer */
	ldr	r0, =SWTCSRA
	ldr	r1, =0xa5a5a500
	str	r1, [r0]

	/****************************************************************
	 * main
	 ****************************************************************/
	ldr	r0, =ICRAM0_IPL_START
	blx	entry /* see entry.c */

1:	b	1b
