/*
 * Initial C entry point of the boot loader.
 * Called by the assembly language setup code in boot.S.
 *
 * Copyright (c) 2014 Atmark Techno, Inc.
 */

#include "types.h"

/* see ipl.lds */
extern unsigned char __entry_start_src;
extern unsigned char __entry_start;
extern unsigned char __entry_end;
extern unsigned char __text_start_src;
extern unsigned char __text_start;
extern unsigned char __text_end;
extern unsigned char __data_start_src;
extern unsigned char __data_start;
extern unsigned char __data_end;
extern unsigned char __bss_start;
extern unsigned char __bss_end;

/* see ipl.c */
extern void ipl_main(void);

typedef void (*entry_t)(void);

void entry(addr_t) __attribute__((__section__(".pre.entry")));
void entry(addr_t entry_point)
{
	entry_t entry_func = (entry_t)(entry_point + 1);
	unsigned char *src, *dst;

	/* relocate .entry */
	src = &__entry_start_src;
	dst = &__entry_start;
	while (dst < &__entry_end)
		*dst++ = *src++;

	entry_func();
}

void entry_post(void) __attribute__((__section__(".post.entry")));
void entry_post(void)
{
	unsigned char *src, *dst;

	/* relocate .text */
	src = &__text_start_src;
	dst = &__text_start;
	while (dst < &__text_end)
		*dst++ = *src++;

	/* relocate .data */
	src = &__data_start_src;
	dst = &__data_start;
	while (dst < &__data_end)
		*dst++ = *src++;

	/* clear .bss */
	dst = &__bss_start;
	while (dst < &__bss_end)
		*dst++ = 0;

	ipl_main();
}
